<?php
//-------------------------------------------------------------------------
// OVIDENTIA http://www.ovidentia.org
// Ovidentia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//-------------------------------------------------------------------------
/**
 * @license http://opensource.org/licenses/gpl-license.php GNU General Public License (GPL)
 * @copyright Copyright (c) 2006 by CANTICO ({@link http://www.cantico.fr})
 */
require_once 'base.php';
require_once dirname(__FILE__).'/functions.php';



/**
 * Get configuration form
 * @return Widget_Form
 */
function portlets_getPreferencesForm(portlet_PortletDefinitionInterface $portletDefinition)
{
    $W = portlets_Widgets();
    $form = $W->Form(null, $W->VBoxLayout()->setVerticalSpacing(.6, 'em'));
    $form->setName('preferences');

    $preferences = $portletDefinition->getPreferenceFields();
    $nofield = true;

    array_unshift($preferences, array('name' => '_blockTitle', 'label' => portlets_translate('Block title'), 'type' => 'text'));

    foreach ($preferences as $field)
    {
        $label = null;
        $input = null;
        $layout = null;

        if (isset($field['label'])){
            $label = $W->Label($field['label']);
        }

        switch ($field['type']) {
            default:
            case 'text':
                $input = $W->LineEdit();
                break;

            case 'link':
                $label = $W->Link($field['text'], $field['link']);
                break;

            case 'int':
                $input = $W->RegExpLineEdit()
                    ->setRegExp('^[0-9]+$')
                    ->setSize(5);
                break;

            case 'boolean':
                $input = $W->Checkbox();
                $layout = $W->HBoxItems($input, $label)
                    ->setVerticalAlign('middle')
                    ->setHorizontalSpacing(4, 'px');
                break;

            case 'password':
                $input = $W->LineEdit()->obfuscate();
                break;

            case 'range':
                $input = $W->Select();
                for ($i = (int)$field['min']; $i <= (int)$field['max']; $i += $field['step']) {
                    $input->addOption($i, $i);
                }
                break;

            case 'list':
                $input = $W->Select();
                if (isset($field['options'])) {
                    foreach ($field['options'] as $opt) {
                        $group = isset($opt['group']) ? $opt['group'] : Widget_Select::NOGROUPKEY;
                        $input->addOption($opt['value'], $opt['label'], $group);
                    }
                }
                break;

            case 'multiselect':
                $input = $W->Multiselect();
                if (isset($field['options'])) {
                    foreach ($field['options'] as $opt) {
                        $group = isset($opt['group']) ? $opt['group'] : Widget_Select::NOGROUPKEY;
                        $input->addOption($opt['value'], $opt['label'], $group);
                    }
                }
                break;

            case 'hidden':
                // $input = $W->Hidden();
                continue 2;

            case 'idTopic':
                $input = $W->TopicPicker();
                break;

            case 'idCategory':
                $input = $W->TopicCategoryPicker();
                break;

            case 'date':
                $input = $W->DatePicker();
                break;

            case 'color':
                $input = $W->ColorPicker();
                break;

            case 'bab_file':
                $input = $W->BabFilePicker()
                    ->addClass('widget-fullwidth');
                break;

            case 'sitemapnode':
                $input = $W->SitemapItemPicker();
                break;

        }

        if (isset($input))
        {
            $input->setName($field['name']);
        }

        if (isset($label) && isset($input))
        {
            $label->setAssociatedWidget($input);
        }

        if (!isset($layout))
        {
            $layout = $W->FlowItems($label, $input);
            $layout->setHorizontalSpacing(1, 'em');
        }


        $form->addItem($layout);
        $nofield = false;

    }

    if ($nofield)
    {
        return null;
    }

    return $form;
}








