<?php
//-------------------------------------------------------------------------
// OVIDENTIA http://www.ovidentia.org
// Ovidentia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//-------------------------------------------------------------------------
/**
 * @license http://opensource.org/licenses/gpl-license.php GNU General Public License (GPL)
 * @copyright Copyright (c) 2006 by CANTICO ({@link http://www.cantico.fr})
 */

require_once dirname(__FILE__).'/node.ui.php';

bab_Widgets()->includePhpClass('Widget_Form');






class smed_RightEdit extends Widget_Form
{
    /**
     * @var smed_Node
     */
    private $node;


    public function __construct(smed_Node $node) {
        $W = bab_Widgets();

        $layout = $W->VBoxLayout()->setVerticalSpacing(1, 'em');
        parent::__construct(null, $layout);

        $this->node = $node;
        
        $this->setName('node');
        $this->addClass('BabLoginMenuBackground');
        $this->addClass('widget-bordered');

        $this->setHiddenValue('tg', bab_rp('tg'));
        


        $this->addFields();
        $this->addButtons();

        $this->loadValues();
    }

    


    

    protected function addButtons()
    {
        $W = bab_Widgets();
        /* @var $I Func_Icons */
        $I = \bab_Functionality::get('Icons');
        if ($I) {
            $I->includeCss();
        }

        $controller = smed_controller();
        $nodeId = $this->node->getId();
        

        $buttons = $W->FlowItems(
            $W->SubmitButton()->validate()
            ->setAction($controller->Node()->saveRights())
            ->setSuccessAction($controller->Node()->display($nodeId))
            ->setFailedAction($controller->Node()->editRights($nodeId))
            ->setLabel(smed_translate('Save'))
            )->addClass(\Func_Icons::ICON_LEFT_16)->setHorizontalSpacing(1, 'em');


        $this->addItem($buttons);
    }



    protected function loadValues()
    {
    
        $this->setHiddenValue('node[id_function]', $this->node->getId());
        $values = array();
        $contenttypes = $this->node->getContentTypes();
        $values['contenttypes'] = array();
        foreach($contenttypes as $checked) {
            $values['contenttypes'][$checked] = 1;
        }
        require_once $GLOBALS['babInstallPath'].'admin/acl.php';
        $values['create'] = $this->node->getCreate();
        $values['read'] = $this->node->getRead();
        $values['update'] = $this->node->getUpdate();
        $values['delete'] = $this->node->getDelete();
        
        $this->setValues(array('node' => $values));
    
    }


    protected function addFields()
    {
        $W = bab_Widgets();
        
        $hbox = $W->HBoxItems(
            $col1 = $W->VBoxLayout()->setVerticalSpacing(1, 'em'),
            $col2 = $W->VBoxLayout()->setVerticalSpacing(1, 'em')
        )->setHorizontalSpacing(4, 'em');
        
        $this->addItem($hbox);
        
        $col1->addItem($this->recursive());
        $col2->addItem($this->contenttypes());
        
        if ($this->node->canEditRights()) {
            $col1->addItem($this->create());
            $col1->addItem($this->read());
            $col1->addItem($this->update());
            $col1->addItem($this->delete());
        }
    }
    
    

    
    
    protected function recursive()
    {
        $W = bab_Widgets();
    
        return $W->LabelledWidget(
            smed_translate('Set right recursively on childnodes'),
            $W->CheckBox(),
            __FUNCTION__
        );
    }
    
    
    protected function contenttypes()
    {
        $W = bab_Widgets();
        
        $vbox = $W->VBoxLayout()->setVerticalSpacing(0.5, 'em');
        $frame = $W->Frame(null, $vbox);
        $frame->setName(__FUNCTION__);
        $vbox->addItem($W->Title(smed_translate('Allowed content types')));
        
        
        $defaultFunc = bab_functionality::get('SitemapEditorNode');
        /*@var $defaultFunc Func_SitemapEditorNode */
         
        foreach($defaultFunc->getAllContentTypes() as $content_type => $arr) {
        
            $vbox->addItem(
                $W->LabelledWidget($arr['description'], $W->CheckBox(), $content_type)
            );
        }
        
        return $frame;
    }
    
    
    protected function create()
    {
        $W = bab_Widgets();

        return $W->Acl()->setName(__FUNCTION__)->setTitle(smed_translate('Who can create sub-nodes?'));
    }
    
    protected function read()
    {
        $W = bab_Widgets();
    
        return $W->Acl()->setName(__FUNCTION__)->setTitle(smed_translate('Who can view?'));
    }
    
    
    protected function update()
    {
        $W = bab_Widgets();
    
        return $W->Acl()->setName(__FUNCTION__)->setTitle(smed_translate('Who can modify node?'));
    }
    
    protected function delete()
    {
        $W = bab_Widgets();
    
        return $W->Acl()->setName(__FUNCTION__)->setTitle(smed_translate('Who can delete the node?'));
    }
}