<?php
//-------------------------------------------------------------------------
// OVIDENTIA http://www.ovidentia.org
// Ovidentia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//-------------------------------------------------------------------------
/**
 * @license http://opensource.org/licenses/gpl-license.php GNU General Public License (GPL)
 * @copyright Copyright (c) 2008 by CANTICO ({@link http://www.cantico.fr})
 */
include_once 'base.php';
bab_Widgets()->includePhpClass('Widget_Form');


class storeclient_OptionsEditor extends Widget_Form
{
	public function __construct()
	{
		$W = bab_Widgets();

		parent::__construct(null, $W->VBoxLayout()->setVerticalSpacing(1, 'em'));


		$this->setName('options');
		$this->addClass('widget-bordered');
		$this->addClass('BabLoginMenuBackground');
		$this->addClass('widget-centered');
		$this->colon();

		$this->setCanvasOptions($this->Options()->width(60, 'em'));

		$this->addItems();
		$this->loadFormValues();

		$this->addButtons();
		$this->setSelfPageHiddenFields();
	}



	protected function addItems()
	{
		$W = bab_Widgets();


		$this->addItem(
		    $W->LabelledWidget(
				storeclient_translate('Store url'),
				$W->LineEdit()->setSize(40),
				'list_url',
				storeclient_translate('This url must point to a json-encoded file with the available packages')
            )
        );

		/*
		$this->addItem($W->LabelledWidget(
				storeclient_translate('Root archives folder url'),
				$W->LineEdit()->setSize(40),
				'root_url'
		));
		*/

		$this->addItem(
		    $W->LabelledWidget(
				storeclient_translate('Automatically upgrade the installed packages on a daily scheduled task'),
				$W->CheckBox(),
				'autoupgrade'
            )
        );
	}


	protected function loadFormValues()
	{
		$registry = bab_getRegistry();
		$registry->changeDirectory('/bab/install_repository/');
		$values = array(
			'list_url' => (string) $registry->getValue('list_url'),
			'root_url' => (string) $registry->getValue('root_url'),
			'autoupgrade' => (bool) $registry->getValue('autoupgrade')
		);
		$this->setValues($values, array('options'));
	}


	protected function addButtons()
	{
		$W = bab_Widgets();

		$button = $W->FlowItems(
			$W->SubmitButton()
				->setAction(storeclient_Controller()->Admin()->saveConfiguration())
				->setSuccessAction(storeclient_Controller()->Admin()->configure())
				->setName('save')->setLabel(storeclient_translate('Save'))
		)->setSpacing(1, 'em');

		$this->addItem($button);
	}
}





class storeclient_InstallList {


	public function CoreFrame()
	{
		$repository = bab_getInstance('bab_InstallRepository');
		/*@var $repository bab_InstallRepository */

		$file = $repository->getLastest('ovidentia');


		if (!isset($file)) {
			return null;
		}

		$recommended = $this->getRecommendedVersion($file);
        $currentVersion = $file->getCurrentVersion();


		$W = bab_Widgets();
		$frame = $W->Frame(null, $W->HBoxLayout())
		->addClass('BabLoginMenuBackground')
		->addClass('widget-bordered')
		->addClass(Func_Icons::ICON_LEFT_24);

		$frame->addItem(
			$W->VBoxItems(
				$W->Label('Ovidentia')->addClass('widget-strong'),
				$W->Label(sprintf(storeclient_translate('Current installed version: %s'), $this->version($currentVersion))),
				$W->Label(sprintf(storeclient_translate('Last available version: %s'), $this->version($file->version)))
			)->setSizePolicy(Widget_SizePolicy::MAXIMUM)
		);

		if ($file->isInstalled() && !$file->isUpgradable()) {
			$frame->addItem($W->Label(storeclient_translate('Up to date'))->addClass('widget-nowrap'));
		} else {
			$frame->addItem(
				$links = $W->VBoxLayout()->setVerticalSpacing(2,'em')
			);
			

			if ($recommended->version != $file->version) {
			    
			    if (version_compare($currentVersion, $recommended->version, '<')) {
    				$links->addItem(
    					$W->VBoxItems(
    						$W->Link(
    							$W->Icon(sprintf(storeclient_translate('Upgrade to the recommended version %s'), $recommended->version), Func_Icons::ACTIONS_DOCUMENT_DOWNLOAD),
    							storeclient_Controller()->Admin()->install(array($recommended->name.'|'.$recommended->version => 1))
    						)->addClass('widget-nowrap'),
    						$W->Label(storeclient_translate('This version contains only bug fixes released since your last update'))
    					)
    				);
			    } else {
			        $links->addItem($W->Label(storeclient_translate('The recommanded version is allready installed')));
			    }
			}

			$last_version_message = '';

			$links->addItem(
				$W->VBoxItems(
					$W->Link(
						$W->Icon(storeclient_translate('Upgrade to the last version'), Func_Icons::ACTIONS_DOCUMENT_DOWNLOAD),
						storeclient_Controller()->Admin()->install(array($file->name.'|'.$file->version => 1))
					)->addClass('widget-nowrap'),
					$W->Label($last_version_message)
				)
			);
		}

		return $frame;
	}



	private function version($version)
	{
		$c_digits = explode('.', $version);
		$beta = (int) end($c_digits);
		if ($beta >= 90) {
			$betaVersion = 1 + $c_digits[1];
			return $c_digits[0] . '.' . $betaVersion . '.0 Beta ' . ($beta - 89) . ' (' . $version . ')';
		}

		return $version;
	}



	private function getRecommendedVersion(bab_InstallRepositoryFile $lastarchive)
	{
		$current_version = $lastarchive->getCurrentVersion();
		
		$c_digits = explode('.', $current_version);

		if (((int) end($c_digits)) >= 90 || count($c_digits) != 3) {
			return $lastarchive;
		}

		$repository = bab_getInstance('bab_InstallRepository');
		/*@var $repository bab_InstallRepository */


		$arr = $repository->getAvailableVersions($lastarchive->name);
		$maxInBranch = '0.0.0';
		
		foreach ($arr as $available_version) {
			$a_digits = explode('.', $available_version);

			if (((int) end($a_digits)) >= 90) {
				continue; // beta version
			}

			if ($a_digits[0] !== $c_digits[0] || $a_digits[1] !== $c_digits[1]) {
				continue; // not the same branch
			}
			
			if (version_compare($maxInBranch, $available_version, '<')) {
			    $maxInBranch = $available_version;
			}
		}
		
		if ('0.0.0' === $maxInBranch) {
		    return null;
		}
		
		return $repository->getFile($lastarchive->name, $maxInBranch);
	}



	public function AddonsTableView()
	{
		$W = bab_Widgets();
		$table = $W->BabTableView();
		$table->addClass('storeclient-addons-list');

		$row = 0;
		$table->addHeadRow($row);
		$table->addItem($W->Label(storeclient_translate('Name')), $row, 0);
		$table->addItem($W->Label(storeclient_translate('Current version')), $row, 1);
		$table->addItem($W->Label(storeclient_translate('Last available version')), $row, 2);
		$table->addItem($W->Label(storeclient_translate('Install')), $row, 3);
		$row++;

		$repository = bab_getInstance('bab_InstallRepository');
		/*@var $repository bab_InstallRepository */

		try {
			$files = $repository->getFiles();
		} catch (Exception $e) {
			throw new bab_AccessException($e->getMessage());
		}

		foreach ($files as $file) {
			/*@var $file bab_InstallRepositoryFile */
			if ($file->name === 'ovidentia') {
				continue;
			}
			try {
				$current_version = (string) $file->getCurrentVersion();
			} catch (Exception $e) {
				// addon configuration in database but addonini file does not exists
				// this will allow use to restore the files as if the addon was never installed
				$current_version = '';
			}

			$table->addItem(
			    $W->VBoxItems(
				    $W->Label($file->name)->addClass('widget-strong'),
				    $W->Label($file->description)
                ),
			    $row,
			    0
			);

			$table->addItem($W->Label($current_version), $row, 1);
			$table->addItem($W->Label($file->version), $row, 2);

			if ($file->isInstalled() && !$file->isUpgradable()) {
				$table->addItem($W->Label(storeclient_translate('Up to date')), $row, 3);
			} else {
				$table->addItem($W->CheckBox()->setName($file->name.'|'.$file->version), $row, 3);
			}
			$row++;
		}

		return $table;
	}
}
