<?php
//-------------------------------------------------------------------------
// OVIDENTIA http://www.ovidentia.org
// Ovidentia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//-------------------------------------------------------------------------
/**
 * @license http://opensource.org/licenses/gpl-license.php GNU General Public License (GPL)
 * @copyright Copyright (c) 2011 by CANTICO ({@link http://www.cantico.fr})
 */

/**
 * standalone script to serve addons in a csv file
 * used by the storeclient ovidentia addon
 */

ini_set('display_errors', 0);
error_reporting(E_ALL);
define('STORESERVER_ROOT', '.');
define('STORESERVER_CACHE', 'storeserver.cache');


class storeserver 
{
    private $modules;
    
    
    private function getfile($path)
    {
    	$ext = strrchr($path, '.');
    
    	if ('.zip' !== $ext) {
    		return;
    	}
    	
    	if (!is_readable($path)) {
    		return false;
    	}
    	
    	$zip = new ZipArchive;
    	if (false === $zip->open($path)) {
    		return false;
    	}
    	
    	
    	$ini = $zip->getFromName('programs/addonini.php');
    	
    	if (!$ini) {
    		// not an addon
    		
    		$ini = $zip->getFromName('ovidentia/version.inc');
    		
    		if (!$ini) {
    			// not ovidentia version
    			return false;
    		}
    	}
    	
    	$zip->close();
    	
    	
    	
    	$conf = parse_ini_string($ini, true);
    	
    	if (!$conf) {
    		return false;
    	}
    	
    	$general = $conf['general'];
    	$encoding = isset($general['encoding']) ? $general['encoding'] : 'ISO-8859-15';
    	
    	if (isset($conf['addons'])) {
    		$dependencies = array();
    		foreach ($conf['addons'] as $name => $v) {
    			$f = substr($v, 0, 1);
    			if ('>' != $f && '<' != $f && '=' != $f) {
    				$v = '>='.$v;
    			}
    			$dependencies[$name] = $v;
    		}
    		
    	} else {
    		$dependencies = array();
    	}
    	
    	$name = $general['name'];
    	$relative = substr($path, strlen(STORESERVER_ROOT));
    	$version = $general['version'];
    	$description = isset($general['description.fr']) ? $general['description.fr'] : $general['description'];
    	
    	if ('UTF-8' !== $encoding) {
    		$name = mb_convert_encoding($name, 'UTF-8', $encoding);
    		$description = mb_convert_encoding($description, 'UTF-8', $encoding);
    	}
    	
    	if (!isset($this->modules[$name]))
    	{
    	    $this->modules[$name] = array();
    	}
    	
    	$this->modules[$name][] = array(
    	    'relativePath' => $relative,
    	    'version' => $version,
    	    'description' => $description,
    	    'dependencies' => $dependencies
    	);
    }
    
    
    
    /**
     * Output rows found in the given directory
     * @param string $path
     */
    private function getlist($path)
    {
    	$files = scandir($path);
    	foreach ($files as $value) {
    		if ($value === '.' || $value === '..') {
    			continue;
    		}
    		
    		if (is_dir("$path/$value")) {
    		    $this->getlist("$path/$value");
    		} else {
    			$this->getfile("$path/$value");
    		}
    	}
    }
    
    
    /**
     * Get last update timestamp.
     * @param string $path
     * @return int  The time is returned as a Unix timestamp.
     */
    private function getLastUpdate($path)
    {
    	$files = scandir($path);
    	$lastUpdate = 0;
    	
    	foreach ($files as $value) {
    		if ($value === '.' || $value === '..') {
    			continue;
    		}
    	
    		if (is_dir("$path/$value")) {
    			$ts = $this->getLastUpdate("$path/$value");
    		} else {
    			$ts = filemtime("$path/$value");
    		}
    		
    		if ($lastUpdate < $ts) {
    			$lastUpdate = $ts;
    		}
    	}
    	
    	return $lastUpdate;
    }
    
    /**
     * @return string
     */
    public function getJson()
    {
        if (!isset($this->modules))
        {
            $this->modules = array();
            $this->getlist(STORESERVER_ROOT);
        }
        
        return json_encode($this->modules);
    }
    
    
    public function getCached()
    {
    	$cache = STORESERVER_ROOT.'/'.STORESERVER_CACHE;
    	if (!file_exists($cache) || filemtime($cache) < $this->getLastUpdate(STORESERVER_ROOT)) {
    		$data = $this->getJson();
    		file_put_contents($cache, $data);
    	} else {
    		$data = file_get_contents($cache);
    	}
    	
    	echo $data;
    }


}


$storeserver = new storeserver;

header('Content-type:application/json; charset=utf-8');

//echo $storeserver->getJson();
echo $storeserver->getCached();
